//////////////////////////////////////////////////////////////////////
//
// Module    : ProjectCCSettingsWrapper.cpp
// Author    : Krasimir Angelov <kr.angelov@gmail.com>
// Copyright : (c) Krasimir Angelov, All Rights Reserved
//
// implementation of the CProjectCCSettingsWrapper class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Resource.h"
#include "ProjectCCSettingsWrapper.h"

OBJECT_ENTRY_AUTO(CLSID_CCPropertyPage, CCCPropertyPage)

// =========================================================================
//                                                 construct, init, destruct
/* static */
HRESULT CProjectCCSettingsWrapper::CreateInstance(CCCPropertyPage *pPage, IUnknown *pUnkParent, IUnknown **pIUnkPropPage)
{
	HRESULT hr = S_OK;
	CComObject<CProjectCCSettingsWrapper> * pObj;
	hr = CComObject<CProjectCCSettingsWrapper>::CreateInstance(&pObj);
    if (FAILED(hr))
		return hr;
	if (!pObj)
		return E_UNEXPECTED;

    pObj->AddRef();
    hr = pObj->Init(pPage, pUnkParent, pIUnkPropPage);
    pObj->Release();
    return hr;
}


CProjectCCSettingsWrapper::CProjectCCSettingsWrapper()
{
	m_srpIProjectCCSettings = NULL;
	m_cbstrOptions = "";
	m_cbstrIncludeDirectories = "";
	m_cbstrIncludes = "";

	m_pPage = NULL;
}


HRESULT CProjectCCSettingsWrapper::Init(CCCPropertyPage *pPage, IUnknown *pUnkParent, IUnknown **pIUnkPropPage)
{
	HRESULT hr;

    if (!pUnkParent || !pIUnkPropPage)
		return E_UNEXPECTED;

	// initialize the members
    hr = pUnkParent->QueryInterface(__uuidof(IProjectCCSettings), (void**) &m_srpIProjectCCSettings);
    if (FAILED(hr))
		return hr;

    hr = m_srpIProjectCCSettings->get_IncludeDirectories(&m_cbstrIncludeDirectories);
    if (FAILED(hr))
		return hr;

    hr = m_srpIProjectCCSettings->get_Includes(&m_cbstrIncludes);
    if (FAILED(hr))
		return hr;

    hr = m_srpIProjectCCSettings->get_Options(&m_cbstrOptions);
    if (FAILED(hr))
		return hr;

	m_pPage = pPage;

	*pIUnkPropPage = NULL;
    return QueryInterface(IID_IUnknown, (void**) pIUnkPropPage);
}


CProjectCCSettingsWrapper::~CProjectCCSettingsWrapper()
{
}
//                                                 construct, init, destruct
// =========================================================================


// =========================================================================
//                                                      IPerPropertyBrowsing
// Used to populate Debug Start Mode list box

STDMETHODIMP CProjectCCSettingsWrapper::GetDisplayString(
    /* [in] */ DISPID dispID,
    /* [out] */ BSTR *pBstr)
{
	if(pBstr == NULL )
		return E_POINTER;
	*pBstr = NULL;

    return E_NOTIMPL;
}


STDMETHODIMP CProjectCCSettingsWrapper::MapPropertyToPage(
    /* [in]  */ DISPID dispID,
    /* [out] */ CLSID *pClsid)
{
    return E_NOTIMPL;
}


STDMETHODIMP CProjectCCSettingsWrapper::GetPredefinedStrings(
    /* [in]  */ DISPID          dispID,
    /* [out] */ CALPOLESTR *    pCaStringsOut,
    /* [out] */ CADWORD *       pCaCookiesOut)
{
	return E_NOTIMPL;
}


STDMETHODIMP CProjectCCSettingsWrapper::GetPredefinedValue(
    /* [in]  */ DISPID dispID,
    /* [in]  */ DWORD dwCookie,
    /* [out] */ VARIANT *pVarOut)
{
    return E_NOTIMPL;
}
//                                                      IPerPropertyBrowsing
// =========================================================================


// =========================================================================
//                                                    IVsPerPropertyBrowsing

STDMETHODIMP CProjectCCSettingsWrapper::HideProperty(DISPID dispid, BOOL* pfHide)
{
    if (!pfHide)
        return E_INVALIDARG;
    return E_NOTIMPL;
}


STDMETHODIMP CProjectCCSettingsWrapper::DisplayChildProperties(DISPID dispid, BOOL* pfDisplay)
{
    if (!pfDisplay)
        return E_INVALIDARG;
    return E_NOTIMPL;
}


STDMETHODIMP CProjectCCSettingsWrapper::GetLocalizedPropertyInfo(
    DISPID dispid,
    LCID localeID,
    BSTR* pbstrLocalizedName,
    BSTR* pbstrLocalizedDescription)
{
    return E_NOTIMPL;
}


STDMETHODIMP CProjectCCSettingsWrapper::HasDefaultValue(DISPID dispid, BOOL* fDefault)
{
    return E_NOTIMPL;
}


STDMETHODIMP CProjectCCSettingsWrapper::IsPropertyReadOnly(DISPID dispid, BOOL* pfReadOnly)
{
    if (!pfReadOnly)
        return E_INVALIDARG;
    return E_NOTIMPL;
}


STDMETHODIMP CProjectCCSettingsWrapper::GetClassName(BSTR*)
{
	return E_NOTIMPL;
}


STDMETHODIMP CProjectCCSettingsWrapper::CanResetPropertyValue(DISPID , BOOL* )
{
	return E_NOTIMPL;
}


STDMETHODIMP CProjectCCSettingsWrapper::ResetPropertyValue(DISPID )
{
	return E_NOTIMPL;
}

HRESULT WINAPI CProjectCCSettingsWrapper::get_Options(BSTR *pValue)
{
	return m_cbstrOptions.CopyTo(pValue);
}

HRESULT WINAPI CProjectCCSettingsWrapper::put_Options(BSTR Value)
{
	HRESULT hr = m_cbstrOptions.AssignBSTR(Value);
	if (FAILED(hr))
		return hr;

	m_pPage->SetDirty(TRUE);
	return S_OK;
}

HRESULT WINAPI CProjectCCSettingsWrapper::get_IncludeDirectories(BSTR *pValue)
{
	return m_cbstrIncludeDirectories.CopyTo(pValue);
}

HRESULT WINAPI CProjectCCSettingsWrapper::put_IncludeDirectories(BSTR Value)
{
	HRESULT hr = m_cbstrIncludeDirectories.AssignBSTR(Value);
	if (FAILED(hr))
		return hr;

	m_pPage->SetDirty(TRUE);
	return S_OK;
}

HRESULT WINAPI CProjectCCSettingsWrapper::get_Includes(BSTR *pValue)
{
	return m_cbstrIncludes.CopyTo(pValue);
}

HRESULT WINAPI CProjectCCSettingsWrapper::put_Includes(BSTR Value)
{
	HRESULT hr = m_cbstrIncludes.AssignBSTR(Value);
	if (FAILED(hr))
		return hr;

	m_pPage->SetDirty(TRUE);
	return S_OK;
}

STDMETHODIMP CProjectCCSettingsWrapper::Commit()
{
	HRESULT hr;

    hr = m_srpIProjectCCSettings->put_IncludeDirectories(m_cbstrIncludeDirectories);
    if (FAILED(hr))
		return hr;

    hr = m_srpIProjectCCSettings->put_Includes(m_cbstrIncludes);
    if (FAILED(hr))
		return hr;

    hr = m_srpIProjectCCSettings->put_Options(m_cbstrOptions);
    if (FAILED(hr))
		return hr;

	return S_OK;
}

//                                                    IVsPerPropertyBrowsing
// =========================================================================
